/**
 * @fileoverview 指定のXML要素の位置に、テキストフレームをコピーペーストし、指定のXMLコンテンツをペーストする
 * @author Kenshi Muto &lt;kmuto@debian.org&gt;
 * @requires libCommon.jsx
 * @requires glue code.jsx
*/

/*
    Copyright:
    2014 Kenshi Muto <kmuto@debian.org>
----------------------------------------------------------------------
ソフトウェア使用許諾同意書
本ソフトウェアの利用・変更・再配布にあたっては、下記の使用許諾同意書に
同意する必要があります。

1. 本使用許諾同意書における「ソフトウェア」とは、機械可読の資料 (ライブ
   ラリ、スクリプト、ソースファイル、データファイル)、実行形式、および
   文書を意味します。
2. 本ソフトウェアの使用許諾同意書に同意する限りにおいて、使用者は
   本ソフトウェアを自由に利用、変更することができます。
3. 本ソフトウェアに変更を加えない限りにおいて、使用者は本ソフトウェアを
   自由にコピー、再配布することができます。
4. 本ソフトウェアは無保証です。作者およびそれに関連する組織、配布者は、
   本ソフトウェアの使用に起因する一切の直接損害、間接損害、偶発的損害、
   特別損害、懲戒的損害、派生的損害について何らの責任・保証も負いません。
5. 本ソフトウェアを変更した上で再配布するときには、下記の事項すべてに
   従わなければなりません。
   - 使用許諾同意書の内容に変更を加えてはなりません。技術上の理由で
     文字エンコーディングの変換を行うことは許可しますが、その使用者が
     特殊な技術的措置なしに可読な形でなければなりません。
   - 技術上の理由でバイナリ化・難読化を行う場合も、変更箇所を含めた
     ソフトウェアを、その使用者が可読可能な形式の形で同一のメディアで
     提供しなければなりません。本使用許諾同意書の2条および3条により、
     使用者が可読形式の該当ソフトウェアを変更、コピー、再配布することを
     妨げてはなりません。
   - ソフトウェア構成物の所定の作者名の欄に、変更者のクレジット
     (個人名、企業名、所属、連絡先など)を「追加」しなければなりません。
6. 本ソフトウェアを変更した上で再配布するときには、変更理由および
   その内容を明記することが推奨されます。
7. 使用者がソフトウェアに適用可能な特許に対して特許侵害にかかわる何らか
   の行動を開始した時点で、この使用許諾同意書は自動的に終了し、以降
   使用者はこの使用許諾書によって与えられた一切の権利を放棄するものと
   します。

著作権所有者 Copyright (C) 2014 Kenshi Muto. All rights reserved.
使用許諾同意書バージョン1.0
著作権所有者による書面での事前の許可がない限り、この使用許諾同意書
に変更を加えてはなりません。
----------------------------------------------------------------------
*/

#include "../libs/libCommon.jsx"
#include "../libs/glue code.jsx"

/**
 * 中身ペースト用のXML要素を探し、その要素オブジェクトをobj.targets配列に格納する
 * @param {Object} obj パラメータオブジェクト
 * @param {String} obj.targetname 探索するXML要素名
 * @param {String} obj.reftarget 参照するID用属性(デフォルトは"refid")
 * @type Nothing
*/
function processXMLForFindTargetElements(obj) {
  obj.targets = [];
  var myRuleSet = new Array(new processForFindTargetElements(obj));
  __processRuleSet(document.xmlElements.item(0), myRuleSet);
}

/**
 * processXMLForFindTargetElementsから呼び出される実体。中身ペースト用のXML要素を探し、その要素オブジェクトをobj.targets配列に格納する
 * @param {Object} obj パラメータオブジェクト
 * @param {String} obj.targetname 探索するXML要素名
 * @param {String} obj.reftarget 参照するID用属性(デフォルトは"refid")
 * @type Nothing
*/
function processForFindTargetElements(obj) {
  this.name = "ProcessSubset";
  if (obj.reftarget == null) obj.reftarget = "refid";
  if (obj.tagname.match(/^\//) != null) {
    this.xpath = obj.targetname;
  } else {
    this.xpath = "//" + obj.targetname;
  }
  this.apply = function(myElement, myRuleProcessor) {
    obj.targets[myElement.xmlAttributes.item(obj.reftarget).value] = myElement;
    return true;
  }
}

/**
 * XML要素を検索して、指定の要素を発見したらテキストフレームをコピーペーストし、中にobj.refidで指定した属性値と一致するXMLをペーストする
 * @param {Object} obj パラメータオブジェクト
 * @param {String} obj.tagname 対象とするXML要素名
 * @param {String} obj.refid 参照するID用属性(デフォルトは"ref")
 * @param {Object} obj.item コピーペーストするTextFrameオブジェクト
 * @param {XMLElement} obj.targets[] ペースト候補XML要素の一覧
 * @type Nothing
*/
function processXMLForPlaceTextFrame(obj) {
  var myRuleSet = new Array(new processForPlaceTextFrame(obj));
  __processRuleSet(document.xmlElements.item(0), myRuleSet);
}

/**
 * processXMLForPlaceTextFrameから呼び出される実体。XML要素を検索して、指定の要素を発見したらテキストフレームをコピーペーストし、中にobj.refidで指定した属性値と一致するXMLをペーストする
 * @param {Object} obj パラメータオブジェクト
 * @param {String} obj.tagname 対象とするXML要素名
 * @param {String} obj.refid 参照するID用属性(デフォルトは"ref")
 * @param {Object} obj.item コピーペーストするTextFrameオブジェクト
 * @param {XMLElement} obj.targets[] ペースト候補XML要素の一覧
 * @type Nothing
*/
function processForPlaceTextFrame(obj) {
  this.name = "ProcessSubset";
  if (obj.tagname.match(/^\//) != null) {
    this.xpath = obj.tagname;
  } else {
    this.xpath = "//" + obj.tagname;
  }
  this.apply = function(myElement, myRuleProcessor) {
    obj.point = myElement.storyOffset;
    if (obj.refid == null) obj.refid = "ref";
    obj.ref = myElement.xmlAttributes.item(obj.refid).value;
    placeTextFrame(obj);
    return true;
  }
}

/**
 * テキストフレームをコピーペーストし、中にobj.refidで指定した属性値と一致するXMLをペーストする
 * @param {Object} obj パラメータオブジェクト
 * @param {Object} obj.item コピーペーストするTextFrameオブジェクト
 * @param {InsertionPoint} obj.point ペーストする位置
 * @param {XMLElement} obj.targets[] ペースト候補XML要素の一覧
 * @type Nothing
*/
function placeTextFrame(obj) {
  var page;
  try {
    page = obj.point.parentTextFrames[0].parentPage;
  } catch (e) {
    page = obj.point.parentTextFrames[0].parent;
  }

  var copieditem = obj.item.duplicate(page);
  copieditem.placeXML(obj.targets[obj.ref]);

  // 末尾行削除
  var line = copieditem.lines[copieditem.lines.length - 2];
  if (line.characters[line.characters.length - 1].contents == "\r") {
    line.characters[line.characters.length - 1].remove();
  }
  copieditem.fit(FitOptions.frameToContent);

  copieditem.select();
  app.cut();
  obj.point.select();
  app.paste();
}
